/***************************************************************************
 * Include : nivel1.h                 Versin : 4.0.1     FECHA : 24/oct/02
 *
 * Descripcin :
 *   Funciones de interfaz con Level 1 para prcticas de
 *	Redes de Comunicaciones
 *
 * Autor : Raul Wegmann                                   Fecha : 24/may/00
 * Centro: UAM - II
 *
 * Cambios:
 *   * 15/7/2000. V 2.0.0. Antonio Martnez
 *	- Aadida funcin para activar trazas.
 *   * 20/7/2000. V 2.1.0. Antonio Martnez
 *	- Cambio interfaz nueva_trama a Espera_nueva_trama
 *	- Inicializacin con solicitud llamada rutina interrupcin con timer
 *   * 10/10/2000. V 2.1.1. Antonio Martnez
 *      - Especificado intervalo llamada a rut. Interrupcin en ms.
 *	- Cambiado nombre rutina activacin trazas para consistencia con resto
 *          de nombres.
 *	- Cambiado criterio no timeout en Espera_Nueva_Trama a 0 ms.
 *   * 18/10/2000. V 2.1.2. Antonio Martnez
 * 	- Incluidas trazas adicionales para funcin Inic_nivel1
 *	- Incluidos en la librera los mdulos loc_sock.lo y trazas.lo.
 *	- Visualizacin de la versin en la funcin Inic_nivel1
 *   * 17/11/2000. V 2.1.3. Antonio Martnez
 *	- Cambiada comprobacin tamao mximo trama a enviar a
 *	    ETH_FRAME_MAX. Por error era ETH_DATA_MAX.
 *	- Introducido retardo en funcin inicializacin nivel 1 para
 *	    permitir que arranque correctamente la tarea de recepcin de
 *	    tramas.
 *	- Definido tipo para la funcin de interrupcin del nivel 1
 *      - Cambiados tipos de datos para los arrays de char a BYTE.
 *   * 17/01/2001. V 3.0.0. Ruben Cabello
 *	- Implementado modo virtual para un solo ordenador
 *   * 30/07/2001. V 3.0.1. Antonio Martnez
 *      - Integrados cambios V2.1.3 y 3.0.0
 *   * 22/09/2001. V 3.0.2. Antonio Martnez
 *      - Modificada finalizacin nivel 1 con Ctrl-c para evitar bloqueos
 *   * 16/10/2001. V 3.0.3. Antonio Martnez
 *      - Liberacin semforo en DesInici_nivel1 en salida por error
 *      - Nueva remodelacin cierre por Signal.
 *   * 20/09/2002. V 4.0.0. Antonio Martnez
 *      - Cambiados los nombres de las funciones de la interfaz nivel1
 *      - Definidas constantes para los cdigos de retorno
 *   * 24/10/2002. V 4.0.1. Antonio Martnez
 *      - Cambiado texto de la traza de EsperaNuevaTrama
* Definida variable de version ETHD_VERSION y ETHD_VERSION_4_1_0
 *************************************************************************/

#ifndef _nivel1_H_
#define _nivel1_H_

/**************************** Tipos de datos  ****************************/

#ifndef _BYTE_
#define _BYTE_
typedef unsigned char BYTE;
#endif

#ifndef _WORD_
#define _WORD_
typedef unsigned short WORD;
#endif

/* Tipo de datos de Puntero a la funcin de notificacin desde nivel 1   */
typedef int (*tpfNotificacionDesdeNivel1) (int, const BYTE *);

/************************ Definicion de constantes ***********************/

#define ETH_ALEN      6      /* Tamao de direccion ethernet             */
#define ETH_HLEN      14     /* Tamao de cabecera ethernet              */
#define ETH_TLEN      2      /* Tamao del campo ethertype               */
#define ETH_FRAME_MAX 1514   /* Tamao mximo trama ethernet (sin CRC)   */
#define ETH_FRAME_MIN 60     /* Tamao mnimo trama ethernet (sin CRC)   */

/* Tamao mximo y mnimo de los datos de una trama ethernet             */
#define ETH_DATA_MAX  (ETH_FRAME_MAX - ETH_HLEN)
#define ETH_DATA_MIN  (ETH_FRAME_MIN - ETH_HLEN)

/* Cdigos de retorno de las funciones                                   */
#define ETH_OK        0
#define ETH_ERROR     -1
#define ETH_TIMEOUT   -2

#define _ETHD_VERSION_4_1_0
#define _ETHD_VERSION 4_1_0


/************************* Prototipos de funciones ***********************/

/*************************************************************************
 * Funcion: EsperaNuevaTrama
 * Fecha: 24.05.2000
 * Autor: Antonio Martnez
 *
 * Espera a que est disponible una nueva trama para su lectura
 *
 * Entrada:
 *    int timeout: Valor en ms de tiempo transcurrido el cual se devuelve
 *	control aunque no haya llegado ninguna trama.
 *	0 o negativo si no se desea timeout.
 * Salida:
 *   ETH_OK: Trama recibida
 *   ETH_TIMEOUT: Salida por timeout
 ************************************************************************/
int EsperaNuevaTrama(int t);

/*************************************************************************
 * Funcion: IniciarNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que inicializa el nivel 1.
 *
 * Entrada:
 *    int num_types: numero de tipos ethernet en ether_types (0-2)
 *    const BYTE *ether_types: tipos de tramas ethernet que se quiere
 *                             recibir
 *    tpfNotificacionDesdeNivel1 interrupt_func:
 *         funcion de interrupcion que se llamara cada vez que se reciba
 *         una trama (puede ser NULL si nprots == 0)
 *    int timer_intr: Solicitud de llamada a la funcin de
 *	   interrupcin por timer cada timer_intr milisegundos.
 *	   Si se pasa valor 0 o negativo no se producen llamadas por timer.
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int IniciarNivel1(int num_types, const BYTE *ether_types,
                tpfNotificacionDesdeNivel1 interrupt_func, int timer_intr);

/*************************************************************************
 * Funcion: FinalizarNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que finaliza nivel1 y libera todos los recursos que ocupa.
 *
 * Entrada:
 *    ninguna
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int FinalizarNivel1();

/*************************************************************************
 * Funcion: EnviarTramaNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que envia una trama ethernet.
 *
 * Entrada:
 *    int len: Longitud de la trama
 *    const BYTE * data: Trama a transmitir
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int EnviarTramaNivel1(int len, const BYTE *data);

/*************************************************************************
 * Funcion: RecibirTramaNivel1
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que recibe una trama. Solo se debe llamar cuando
 * se haya recibido una trama, indicado por la funcin EsperaNuevaTrama
 *
 * Entrada:
 *    BYTE *buffer: puntero donde se almacenara la trama recibida
 * Salida:
 *   Numero de bytes recibidos: si todo ha ido bien
 *   ETH_ERROR: Error.
 *************************************************************************/
int RecibirTramaNivel1(BYTE *buffer);

/*************************************************************************
 * Funcion: ObtenerDirIP
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que obtiene la direccion IP asociada a la interfaz.
 *
 * Entrada:
 *    BYTE *ifaddr: array donde se almacenara la direccion
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int ObtenerDirIP(BYTE *ifaddr);

/*************************************************************************
 * Funcion: ObtenerMascaraIP
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que obtiene la mascara de red asociada a la interfaz.
 *
 * Entrada:
 *    BYTE *ifmask: array donde se almacenara la mascara
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int ObtenerMascaraIP(BYTE *ifmask);


/*************************************************************************
 * Funcion: ObtenerMTU
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que obtiene la MTU de la interfaz.
 *
 * Entrada:
 *    ninguna
 * Salida:
 *   Valor de la MTU si la operacion se realizo correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int ObtenerMTU();

/*************************************************************************
 * Funcion: ObtenerDirMAC
 * Fecha: 24.05.2000
 * Autor: Raul Wegmann
 *
 * Funcion que obtiene la direccion ethernet de la interfaz.
 *
 * Entrada:
 *    BYTE *ifhwaddr: array donde se almacenara la direccion fisica
 * Salida:
 *   ETH_OK: Operacion realizada correctamente
 *   ETH_ERROR: Error en la realizacion de la operacion
 *************************************************************************/
int ObtenerDirMAC(BYTE *ifhwaddr);

/*************************************************************************
 * Funcion: ActivarTrazas
 * Fecha: 14.07.2000
 * Autor: Antonio Martnez
 *
 * Funcion que activa el modo de depuracion, produciendo trazas de la
 *   operacin
 *
 * Entrada:
 *    int tipo: Tipo de trazas que se pide.
 *      0: Suprimir las trazas.
 *      1: Trazas de tramas que se envan y reciben.
 *      2: Trazas tipo 1, y de llamadas a funciones de nivel1.
 *      3: Trazas tipo 2, y de comunicacin con ethd.
 *    char *fichero: Nombre del fichero donde se realizan las trazas.
 *	Acepta stdout y stderr como posibles valores
 * Salida:
 *    0 si todo ha ido bien y -1 en caso de error
 **************************************************************************/
int ActivarTrazas(int tipo, char * fichero);

#endif  /* _nivel1_H_ */
